@@ -12,7 +12,7 @@ use Mail::Sendmail qw(sendmail);
 use Storable qw(store retrieve);
 use Sys::Hostname qw(hostname);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub new
 {
@@ -48,9 +48,7 @@ sub process
 {
     my $self = shift;
 
-    foreach my $member (qw(data stored changed)) {
-        delete $self->{$member};
-    }
+    delete @$self{qw(data stored)};
 
     $self->_get_data_running;
     $self->_get_data_file;
@@ -62,10 +60,8 @@ sub _init
 {
     my $self = shift;
 
-    if (!-e $self->{Config}->{data_file}) {
-        eval { store({}, $self->{Config}->{data_file}) }
-          or croak "Cannot store to $self->{Config}->{data_file}: $!";
-    }
+    eval { store({}, $self->{Config}->{data_file}) }
+      or croak "Cannot store to $self->{Config}->{data_file}: $!";
 
     my $pkg_tmpl = join '::', (__PACKAGE__, '_template');
     no strict 'refs';
@@ -113,29 +109,33 @@ sub _get_data_running
     }
 
     my $re = qr{
-                      \s*
-         (?:\d+?\:)?  \s+
-         (?:\w+)      \s+
-         (?:(?:\d+)   \s*){5}
+                      \s*?
+         (?:\d+?\:)?  \s+?
+         (?:\w+?)     \s+?
+         (?:(?:\d+?)  \s*?){5}
     }x;
 
-    my ($uid, $res);
+    my $uid;
+
+    my @names = grep {
+      !$self->{excluded}->{$_}
+        ? $_ : ()
+    } @{$self->{Config}->{_field_names}};
 
     local $1;
     while ($output =~ /^($re)$/gm) {
         my $line = $1;
-        if ($line =~ s/^\s*(\d+?)\://) {
+        if ($line =~ /^ \s+? (\d+?)\:/gx) {
             $uid = $1;
         }
-        if ($line =~ s/^\s*(\w+)\s*//) {
+        my $res;
+        if ($line =~ /\G \s+? (\w+)/gx) {
             $res = $1;
         }
-        my @fields = split /\s+/, $line;
-        my @names = grep {
-          !$self->{excluded}->{$_}
-            ? $_ : ()
-        } @{$self->{Config}->{_field_names}};
-
+        my @fields;
+        if ($line =~ /\G \s+ (.*) $/x) {
+            @fields = split /\s+/, $1;
+        }
         push @{$self->{data}{$uid}{$res}},
           { map { $names[$_] => $fields[$_] } (0 .. $#fields) };
     }
@@ -157,8 +157,9 @@ sub _compare_data
     {
         my ($uid, $res, $i) = @_;
 
-        my $data   = sub { $self->{data}{$uid}{$res}->[$i]->{$_[0]}   || 0 };
-        my $stored = sub { $self->{stored}{$uid}{$res}->[$i]->{$_[0]} || 0 };
+        my ($data, $stored) = map {
+          my $type = $_; sub { $self->{$type}{$uid}{$res}->[$i]->{$_[0]} || 0 }
+        } qw(data stored);
 
         return scalar grep { $data->($_) > $stored->($_) } @{$self->{Config}->{monitor_fields}};
     };
@@ -177,7 +178,6 @@ sub _compare_data
 sub _create_report
 {
     my $self = shift;
-
     my ($uid, $res, $index) = @_;
 
     if ($self->{Config}->{_tests}) {
@@ -204,7 +204,6 @@ sub _put_data_file
 sub _prepare_report
 {
     my $self = shift;
-
     my ($uid, $res, $index) = @_;
 
     my @fixed_fields = ($uid, $res) x 2;
@@ -238,40 +237,37 @@ sub _prepare_report
         push @values, [ @v ];
     }
 
+    my %changed;
     foreach my $field (keys %{$mapping[0]->[1]}) {
         if ($mapping[0]->[1]->{$field} != $mapping[1]->[1]->{$field}) {
-            $self->{changed}->{$self->{index}->{$field}} = true;
+            $changed{$self->{index}->{$field}} = true;
         }
     }
 
     my $tmpl = \@values;
-    my $template = $self->{template};
+    my $report = $self->{template};
 
-    while ($template =~ /(\$\S+)/) {
-        unless ($template =~ /(\Q$1\E)$/m) {
-            my $len = length($1) - length do { eval $1 };
-            my $space = ' ' x $len;
-            $template =~ s/(\Q$1\E)/$1$space/;
+    local $1;
+
+    while (my ($var) = $report =~ /(\$\S+)/) {
+        unless ($report =~ /\Q$var\E$/m) {
+            my $len = length($var) - length do { eval $var };
+            $report =~ s/(?<=\Q$var\E)/' ' x $len/e;
         }
-        $template =~ s/(\$\S+)/$1/ee;
+        $report =~ s/(\Q$var\E)/$1/ee;
     }
 
-    while ($template =~ /\((\d+)\)/) {
-        if ($self->{changed}->{$1}) {
-            $template =~ s/\($1\)/  \*/;
-        }
-        else {
-            $template =~ s/\($1\)/   /;
-        }
+    while (my ($pos) = $report =~ /\((\d+)\)/) {
+        my $marked = $changed{$pos} ? '*' : ' ';
+        $report =~ s/\($pos\)/  $marked/;
     }
 
-    return $template;
+    return $report;
 }
 
 sub _send_mail
 {
     my $self = shift;
-
     my ($report) = @_;
 
     my %mail = (
@@ -325,6 +321,8 @@ mail notifications whenever important values change (as defined by the user).
 The recommended usage of this application module is to use the provided
 L<vzwatchd> daemon.
 
+Version 2.5 of the F<user_beancounters> file is supported.
+
 =head1 CONSTRUCTOR
 
 =head2 new
@@ -415,12 +413,19 @@ Location of the data file. Defaults to F<$HOME/vzwatchd.dat>.
 
 =item * Cannot parse anything else than the F<user_beancounters> file.
 
+=item * Only version 2.5 of the F<user_beancounters> file is known to work.
+
 =item * Fixed mail report template.
 
 =item * No more than one mail recipient.
 
 =back
 
+=head1 BUGS & CAVEATS
+
+Note that the sender and recipient mail address options with defaults
+might require adjustment in order for notifications to be delivered.
+
 =head1 SEE ALSO
 
 L<OpenVZ::BC>, L<http://openvz.org>
@@ -434,6 +439,6 @@ Steven Schubiger <schubiger@cpan.org>
 This program is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
 
-See L<http://www.perl.com/perl/misc/Artistic.html>
+See L<http://dev.perl.org/licenses/>
 
 =cut
